/*
   ( PIC16F628A,  7- 2-  )
ver 1.1
autor Alex73
     CCS PICC 4.020 + MPLAB 8.10 + Proteus 7.2sp6
     :
//--------------------------------------------------------------------------------
//    Home Timer -    7- 2-           *
//                                              *
//    Author:       picmaniac                                          *
//    Company:      www.pro-radio.ru                                   *
 http://www.pro-radio.ru/controllers/3435/
//--------------------------------------------------------------------------------

    (DS18S20,   RA0   )
     ,    "00"
           
(/ 10  (.18)          )

  12-        (  eeprom):
                 11  22 :
11:     (  60,    ,     2-8 :)
12:     ,    
13:      (<=25),       
14:   ""       
15:        ,   (<=25)
    (  0            )
16:       ,   (<=25)
17:       ,   (<=25)
18:      (<=25)
19: ""  ,  1  10 (   1  10      9  0 )
20:   (=1 CA  =0 CC  )    (=1 -  ( ), =2 - picmaniac' ( )  )
    ( "01" -    ,       )
21:       ,   (<=25)
22:         ,   (<=25)
      .21=8  .22=12 (.. 80  120        ,
         40%)

         
(      ,         :)
*/

#include <16F628A.h>
#FUSES WDT                      //Watch Dog Timer
#FUSES INTRC_IO                 //Internal RC Osc, no CLKOUT
#FUSES PUT                      //Power Up Timer
#FUSES NOPROTECT                //Code not protected from reading
#FUSES NOBROWNOUT               //No Reset when brownout detected
#FUSES MCLR                     //Master Clear pin enabled
#FUSES NOLVP                    //No low voltage prgming, B3(PIC16) or B5(PIC18) used for I/O
#FUSES NOCPD                    //No EE protection
#use delay(clock=4000000)

#use fast_io(A)                 //    / input  output    TRISA  TRISB
#use fast_io(B)

#ZERO_RAM                       //        

//--------------------------------------------------------------------------------
//  (   eeprom        )
// 11
//     (   ,     2-8 ,    60 :)
#rom  0x2100={2}
int8 gnNumbSecInMin;
const int8 gnAdrNumbSecInMin=0x00;
// 12
//     ,    
#rom  0x2101={10}
int8 gnTimeOnKey;
const int8 gnAdrTimeOnKey=0x01;
// 13
//     ,        ( EEPROM   )
#rom  0x2102={100/10}
int8 gnTimeLongOnKey;
const int8 gnAdrTimeLongOnKey=0x02;
// 14
//   ""       
#rom  0x2103={50}
int8 gnTimeRepeatOnKey;
const int8 gnAdrTimeRepeatOnKey=0x03;
// 15
//       ,   ( EEPROM   )
#rom  0x2104={100/10}
int8 gnTimeWorkingDevice;
const int8 gnAdrTimeWorkingDevice=0x04;
// 16
//       ,   ( EEPROM   )
#rom  0x2105={80/10}
int8 gnTimeWorkingSound;
const int8 gnAdrTimeWorkingSound=0x05;
// 17
//       ,   ( EEPROM   )
#rom  0x2106={1000/100}
int16 gnMaxTimeSleep;
const int8 gnAdrMaxTimeSleep=0x06;
// 18
//      ( EEPROM   )
#rom  0x2107={60/10}
int8 gnTimeTemperOut;
const int8 gnAdrTimeTemperOut=0x07;
// 19
// ""  ,  1  10 (   1  10      9  0 )
#rom  0x2108={4}
int8 gnBrightFigure;
const int8 gnAdrBrightFigure=0x08;
// 20
//   (=1 CA  =0 CC  )    (=1 -  ( ), =2 - picmaniac' ( )  )
//  "01" -    ,       
//  "12" -    ,  ""
#rom  0x2109={12}
int1 glIsCommAnode, glXorIsCommAnode;
int8 gnSound;
const int8 gnAdrTypeIndAndSound=0x09;
// 21
#rom  0x210A={8}
int8 gnConstTimeOutFigure;          //          ( EEPROM   )
const int8 gnAdrConstTimeOutFigure=0x0A;
// 22
#rom  0x210B={12}
int8 gnConstTimeNotOutFigure;       //            ( EEPROM   )
const int8 gnAdrConstTimeNotOutFigure=0x0B;
                                    //     80   120 
                                    // ..       ,      40%

const int16 gnTMR1_01 = 0x10000-500;    //      TMR1,   1 KHz
const int16 gnTMR1_02 = 0x10000-400;    //  1250 Hz
const int16 gnTMR1_03 = 0x10000-300;    //  1660 Hz

const int8 gnTimeReadTemper=80;         //        DS18S20

const int8 gnMaxTimeBetwOnKeyStart=150; //         (  )

//================================================================================
// 

int1 glEnableNextCycle;             //       MAIN ( 1000 -1    TMR2)

int8 gnSTATE;                       //   (=0 - " ", =1 - " ", =2 - " ")

int8 gnDiv10;                       //        MAIN
int8 gnDiv100;                      //        MAIN
int8 gnDiv60;                       //        MAIN

int8 gnDec;                         //    (     )
int8 gnOne;                         //    (     )
int8 gnNumbFigure;                  //   : =2 -   , =1 - 

int8 gnTimeOutFigure;               //     
int8 gnTimeNotOutFigure;            //       
                                    //   40  60    1      40% 
                                    //   20  180 - 1   2    10% 
int16 gnCountTimeOutFigure;         //        

int8 gnCountWorkingDevice;          //      
int8 gnDivToDecSec;                 //         

                                    //      _, _  /
                                    //       100   
                                    // glCurrKey... -    (=1 - , =0 -  )
                                    // glPrevKey... -   (  )
                                    // glKey... -    (   gnTimeOnKey   -  =1)
                                    // glKey...Long... -     (   gnTimeLongOnKey   -  =1)
                                    // gnCountKey... -  ,     
int1 glCurrKeyDecOn, glPrevKeyDecOn, glKeyDecOn, glKeyDecLongOn;
int8 gnCountKeyDecOn;
int1 glCurrKeyOneOn, glPrevKeyOneOn, glKeyOneOn, glKeyOneLongOn;
int8 gnCountKeyOneOn;
int1 glCurrKeyStartOn, glPrevKeyStartOn, glKeyStartOn, glKeyStartLongOn;
int8 gnCountKeyStartOn;
int8 gnSumKeysOn;                   //     (   >1 -    )

int8 gnNumbOnKeyStart;              //          gnMaxTimeBetwOnKeyStart
int8 gnCountTimeBetwOnKeyStart;     //        
int8 gnCommand;                     // ,       :
                                    // =2 -   " " <-> " "
int8 gnValConst;                    //      eeprom

int8 gnTimer;                       //   (   , =gnDec*10+gnOne

int16 gnTMR1;                       //  TMR1 (     TMR1)

int8 gnCountWorkingSound;           //    
int8 gnDivForSound;                 //      (       ,   - )
int8 gnDivForSound2;

int1 glIsSound;                     //   ,  =1 -    TMR1   ,    
int1 glValSound;                    // ,   ,      TMR1  0  1  
int8 gnVarSound;                    //  : =1 -  ( ), =2 - picmaniac',  ( ), =3 -   ,  =4 -    

int16 gnCountTimeSleep;             //  ,     
int1 glIsExtInterrutp;              //     (  1    INT0,     MAIN)

int8 gnCountTimeReadTemper;         //      
int8 gnPathReadTemper;              //   : =0 , =1  , =2 , =3 
int8 gnTemper, gnZnak;              //     ("+"  "-")
int8 gnCountTemperOut;              //       

//--------------------------------------------------------------------------------
//   PIC
const int16 pin_Dec = pin_a2;       //    
const int16 pin_One = pin_a3;       //   
const int16 pin_Dev = pin_a4;       //    ( )
const int16 KeyDec = pin_b4;        //    
const int16 KeyOne = pin_b6;        //    
const int16 KeyStart = pin_b5;      //   /
const int16 InputKeys = pin_b0;     //   
const int16 pin_Sound_1 = pin_a0;   //    
const int16 pin_Sound_2 = pin_a1;   //  

#define TOUCH_PIN pin_a0            //  DS18S20

#byte TRISA=0x85
#bit bit_Dev=TRISA.4                //  ,     <->
#bit bit_Sound_1=TRISA.0            // ,     DS18S20
#bit bit_Sound_2=TRISA.1            // 
#byte TRISB=0x86

//================================================================================
// 
//--------------------------------------------------------------------------------
                                                //    TMR2
#int_TIMER2                                     //    (  1000 -1,
void TIMER2_isr()                               //        main()
{
    glEnableNextCycle=1;
}
//--------------------------------------------------------------------------------
                                                //    TMR1
#int_TIMER1                                     //    
void TIMER1_isr() 
{
    set_timer1(gnTMR1);                         //   
    if (glIsSound==1)
    {                                           //    -     
        glValSound++;                           //    
        output_bit(pin_Sound_1,glValSound);     //     
        glValSound++;
        output_bit(pin_Sound_2,glValSound);     //       
        glValSound++;
    }
}
//--------------------------------------------------------------------------------
                                                //    (    )
#int_EXT                                        //     
void EXT_isr()
{
    gnCountTimeSleep=0;                         //    
    glIsExtInterrutp=1;                         //        MAIN()
}
//================================================================================
// 
//--------------------------------------------------------------------------------
void ClearCountTimeSleep()
//    
//       ,   ,      
// ,   ,       ,     ,
//    WDT   
{
    gnCountTimeSleep=0;                         //    
    disable_interrupts(INT_EXT);                //   
}
//--------------------------------------------------------------------------------
                                                //     (   MAIN(), 
                                                //        MAIN())
void StartWorkingTimer()
{
    gnSTATE=1;                                  //     " "
    
    gnDiv100=0;                                 //     
    gnDiv60=0;
    
    gnTimeOutFigure=gnConstTimeOutFigure;       //   ,     
    gnTimeNotOutFigure=gnConstTimeNotOutFigure; //   

    gnCountTimeOutFigure=0;                     //         
}
//--------------------------------------------------------------------------------
                                                //    
void StopWorkingTimer()
{
    gnSTATE=0;                                  //     " "
    
    gnTimer=0;                                  //   ,     
    gnDec=0;
    gnOne=0;
    
    gnTimeOutFigure=100;                        //     ( 100  0     )
    gnTimeNotOutFigure=0;
    gnCountTimeOutFigure=0;                     //         
}

//--------------------------------------------------------------------------------
                                                //      (   MAIN(),
                                                //        MAIN())
void StartWorkingDevice()
{
    bit_Dev=0;                                  //   ,    ,    
    output_bit(pin_Dev,0);                      //     0
    gnCountWorkingDevice=gnTimeWorkingDevice;   //     
    gnDivToDecSec=0;                            //       
}
//--------------------------------------------------------------------------------
                                                //    
void StopWorkingDevice()
{
    bit_Dev=1;                                  //   ,    ,    
    output_bit(pin_Dev,1);                      //    1 (    ,   )
    gnCountWorkingDevice=0;                     //     
}
//--------------------------------------------------------------------------------
                                                //     (   MAIN(),
                                                //        MAIN())
//--------------------------------------------------------------------------------
void StartWorkingSound_path2()                  //     (..   3-   )
{
    gnDivToDecSec=0;                            //      
    setup_timer_1(T1_INTERNAL | T1_DIV_BY_1);   //    TMR1
    set_timer1(gnTMR1);
    enable_interrupts(INT_TIMER1);              //    TMR1
    gnDivForSound=0;                            //   ,    
}
//--------------------------------------------------------------------------------
                                                //       
void StartWorkingSound()
{
    gnVarSound=gnSound;                         //  
    gnCountWorkingSound=gnTimeWorkingSound;     //     
    StartWorkingSound_path2();
}
//--------------------------------------------------------------------------------
                                                //    
void StopWorkingSound()
{
    gnCountWorkingSound=0;                      //     
    output_bit(pin_Sound_1,0);                  //  0   PIC,   
    output_bit(pin_Sound_2,0);
    glIsSound=0;                                //         TMR1
    setup_timer_1(T1_DISABLED);                 //  TMR1
    disable_interrupts(INT_TIMER1);             //     
}
//--------------------------------------------------------------------------------
                                                //       
                                                // (      )
void StartShortWorkingSound()
{
    gnVarSound=3;                               //   -    
    gnCountWorkingSound=1;                      //    1 
    StartWorkingSound_path2();
}
//--------------------------------------------------------------------------------
                                                //         
                                                //   
void StartDoubleShortWorkingSound()
{
    gnVarSound=4;                               //   -    
    gnCountWorkingSound=3;                      //    3 
    StartWorkingSound_path2();
}
//--------------------------------------------------------------------------------
                                                //   (    PIC   )
void TimerOff()
{
    gnCountTimeSleep=gnMaxTimeSleep;            //        ,
}                                               //     MAIN()
//--------------------------------------------------------------------------------
                                                //    EEPROM
void ReadConstFromEEPROM()
{
// 11
    gnNumbSecInMin=read_eeprom(gnAdrNumbSecInMin);          //    
// 12
    gnTimeOnKey=read_eeprom(gnAdrTimeOnKey);                //       
// 13
    gnTimeLongOnKey=read_eeprom(gnAdrTimeLongOnKey);        //          
    gnTimeLongOnKey=gnTimeLongOnKey*10;
// 14
    gnTimeRepeatOnKey=read_eeprom(gnAdrTimeRepeatOnKey);    //   ""     
// 15
    gnTimeWorkingDevice=read_eeprom(gnAdrTimeWorkingDevice);//       
    gnTimeWorkingDevice=gnTimeWorkingDevice*10;
// 16
    gnTimeWorkingSound=read_eeprom(gnAdrTimeWorkingSound);  //       
    gnTimeWorkingSound=gnTimeWorkingSound*10;
// 17
    gnMaxTimeSleep=read_eeprom(gnAdrMaxTimeSleep);          //       
    gnMaxTimeSleep=gnMaxTimeSleep*100;
// 18
    gnTimeTemperOut=read_eeprom(gnAdrTimeTemperOut);        //   
    gnTimeTemperOut=gnTimeTemperOut*10;
// 19
    gnBrightFigure=read_eeprom(gnAdrBrightFigure);          // ""  
    if (gnBrightFigure>10) gnBrightFigure=10;
// 20
    gnSound=read_eeprom(gnAdrTypeIndAndSound);              //     
    if (gnSound>=10) glIsCommAnode=1;                       //   
    else glIsCommAnode=0;
    glXorIsCommAnode=glIsCommAnode^1;
    gnSound=gnSound%10;                                     //   
    if (gnSound>1) gnSound=2;
    else gnSound=1;
// 21
    gnConstTimeOutFigure=read_eeprom(gnAdrConstTimeOutFigure);          //       
    gnConstTimeOutFigure=gnConstTimeOutFigure*10;
// 22
    gnConstTimeNotOutFigure=read_eeprom(gnAdrConstTimeNotOutFigure);    //         
    gnConstTimeNotOutFigure=gnConstTimeNotOutFigure*10;
}
//--------------------------------------------------------------------------------
                                                //    EEPROM
                                                //         
                                                //      (  case xx )  
void WriteConstToEEPROM()
{
    disable_interrupts(GLOBAL);                 //  
    switch (gnCommand)
    {
        case 11:                                //    
            write_eeprom(gnAdrNumbSecInMin, gnValConst); break;
        case 12:                                //       
            write_eeprom(gnAdrTimeOnKey, gnValConst); break;
        case 13:                                //          
            write_eeprom(gnAdrTimeLongOnKey, gnValConst); break;
        case 14:                                //   ""     
            write_eeprom(gnAdrTimeRepeatOnKey, gnValConst); break;
        case 15:                                //       
            write_eeprom(gnAdrTimeWorkingDevice, gnValConst); break;
        case 16:                                //       
            write_eeprom(gnAdrTimeWorkingSound, gnValConst); break;
        case 17:                                //       
            write_eeprom(gnAdrMaxTimeSleep, gnValConst); break;
        case 18:                                //   
            write_eeprom(gnAdrTimeTemperOut, gnValConst); break;
        case 19:                                // ""  
            write_eeprom(gnAdrBrightFigure, gnValConst); break;
        case 20:                                //     
            write_eeprom(gnAdrTypeIndAndSound, gnValConst); break;
        case 21:                                //       
            write_eeprom(gnAdrConstTimeOutFigure, gnValConst); break;
        case 22:                                //         
            write_eeprom(gnAdrConstTimeNotOutFigure, gnValConst); break;
    }
    enable_interrupts(GLOBAL);                  //  
}
//================================================================================
                                                //  PIC,  1    
void init()
{
    set_tris_b(0b00000001);                     //  /
    set_tris_a(0b11110000);
    port_b_pullups(TRUE);                       //     PORTB
    setup_timer_0(RTCC_INTERNAL);               //  WDT  TMR0 (TMR0    )
    setup_wdt(WDT_1152MS);
    setup_timer_1(T1_DISABLED);                 //  TMR1
    setup_timer_2(T2_DIV_BY_4,249,1);           //  TMR2 (     real time 
                                                // , ..       );
                                                //         1000   
    setup_comparator(NC_NC_NC_NC);              //  
    setup_vref(FALSE);
    enable_interrupts(INT_TIMER2);              //    TMR2

    ReadConstFromEEPROM();

    enable_interrupts(GLOBAL);                  //  
    setup_oscillator(False);

    glEnableNextCycle=0;                        //       TMR2        MAIN()
    gnDiv10=0;                                  //      

    gnNumbFigure=0;                             //    =1,     
    
    StopWorkingTimer();                         //    

    output_bit(pin_Dev,1);                      //    ( , ..     )

//  gnCountTimeSleep=0;                         //   
                                                // 
    TimerOff();                                 //    ,   PIC   
}
//================================================================================
                                                // 
int8 GetFigure(int8 lnFigure)
{
    int8 lnReturn;
    
    switch (lnFigure)
    {
        case 0:   lnReturn=0b10000001; break;
        case 1:   lnReturn=0b11100111; break;
        case 2:   lnReturn=0b00010101; break;
        case 3:   lnReturn=0b01000101; break;
        case 4:   lnReturn=0b01100011; break;
        case 5:   lnReturn=0b01001001; break;
        case 6:   lnReturn=0b00001001; break;
        case 7:   lnReturn=0b11100101; break;
        case 8:   lnReturn=0b00000001; break;
        case 9:   lnReturn=0b01000001; break;
        case 111: lnReturn=0b01110001; break;   // "o"
        case 67:  lnReturn=0b10011001; break;   // "C"
        case 45:  lnReturn=0b01111111; break;   // "-"
        case 124: lnReturn=0b01100111; break;   // "-|"
        case 207: lnReturn=0b10100001; break;   // ""
        case 208: lnReturn=0b00110001; break;   // ""
        default:  lnReturn=0b11111111;
    }

    if (glIsCommAnode==0)
    {
        lnReturn=lnReturn^0b11111111;
    }
    return lnReturn;
}
//--------------------------------------------------------------------------------
                                                //    
void OutFigure()
{
    if (++gnNumbFigure==3) gnNumbFigure=1;      //      1  2
    
    output_bit(pin_Dec,glXorIsCommAnode);       //  ,    
    output_bit(pin_One,glXorIsCommAnode);
    
    gnCountTimeOutFigure++;                     //   

    if (gnCountTimeOutFigure<=gnTimeOutFigure)  //        -  
    {
        if (gnNumbFigure==2)                    //  =2 -  
        {
            output_b(GetFigure(gnOne));         //     ,   
            output_bit(pin_One,glIsCommAnode);  //  1   
        }
        else if (gnNumbFigure==1)               //  =1 -  
        {
            output_b(GetFigure(gnDec));         //    
            output_bit(pin_Dec,glIsCommAnode);
        }
    }
    else if (gnCountTimeOutFigure<=((int16)gnTimeOutFigure+(int16)gnTimeNotOutFigure))
    {                                           // ,        ,
        output_bit(pin_Dec,glXorIsCommAnode);   //  ,    ,   
        output_bit(pin_One,glIsCommAnode);
    }

    if (gnCountTimeOutFigure>=((int16)gnTimeOutFigure+(int16)gnTimeNotOutFigure)) gnCountTimeOutFigure=0; //    ,  
}
//--------------------------------------------------------------------------------
                                                //       
void EndOutFigure()
{
    if ((gnDiv10+1)>=gnBrightFigure)            //    + 1  ""
    {
        output_bit(pin_Dec,glXorIsCommAnode);   //     -  ,   
        output_bit(pin_One,glXorIsCommAnode);
    }
}
//--------------------------------------------------------------------------------
                                                //     -   1 ->    
void TestKeys()
{
    output_bit(pin_Dec,glXorIsCommAnode);       //  ,   
    output_bit(pin_One,glXorIsCommAnode);

    output_b(0xFF);                             //       1
    output_bit(KeyDec,0);                       //        0
    glPrevKeyDecOn=glCurrKeyDecOn;              //   
    if (input(InputKeys)==0) glCurrKeyDecOn=1; else glCurrKeyDecOn=0;   //      0 -   
                                                                        //  1 ( ),  0 ( )

    output_b(0xFF);                             //      
    output_bit(KeyOne,0);
    glPrevKeyOneOn=glCurrKeyOneOn;
    if (input(InputKeys)==0) glCurrKeyOneOn=1; else glCurrKeyOneOn=0;

    output_b(0xFF);                             //     
    output_bit(KeyStart,0);
    glPrevKeyStartOn=glCurrKeyStartOn;
    if (input(InputKeys)==0) glCurrKeyStartOn=1; else glCurrKeyStartOn=0;

    gnSumKeysOn = glCurrKeyDecOn + glCurrKeyOneOn + glCurrKeyStartOn;   //    

    if (gnSumKeysOn!=0) ClearCountTimeSleep();  //       -   
    
    if (glIsExtInterrutp==1)                    //           PIC,
    {                                           //       ,    3-
        if (gnSumKeysOn==0) glIsExtInterrutp=0; //         0 ( )
        else gnSumKeysOn=3;                     //  
    }
    output_bit(pin_Dec,glXorIsCommAnode);       //  ,   
    output_bit(pin_One,glXorIsCommAnode);
    output_b(GetFigure(255));
}
//--------------------------------------------------------------------------------
                                                //      
void TestKeyDec()
{
    if (gnSumKeysOn<=1)                         //       1
    {
        if ((glCurrKeyDecOn==1) && (glPrevKeyDecOn==1))             //          
        {
            if (gnCountKeyDecOn<254) gnCountKeyDecOn++;             //       
                                                                    // (  ,    )
            if (gnCountKeyDecOn==gnTimeOnKey) glKeyDecOn=1;         //     ,   
                                                                    //    ,    
            if (gnCountKeyDecOn==gnTimeLongOnKey) glKeyDecLongOn=1; //     

            if (gnCountKeyDecOn>=gnTimeLongOnKey)                   //       
            {
                gnCountKeyDecOn=gnCountKeyDecOn-gnTimeRepeatOnKey;      //  
                glKeyDecOn=1;                                           //    
            }                                                           // ..        
        }
        else                                    // ,   ,
        {
            gnCountKeyDecOn=0;                  //           
            glKeyDecOn=0;
            glKeyDecLongOn=0;
}}}
//--------------------------------------------------------------------------------
                                                //      
                                                //     TestKeyDec()
void TestKeyOne()
{
    if (gnSumKeysOn<=1)
    {
        if ((glCurrKeyOneOn==1) && (glPrevKeyOneOn==1))
        {
            if (gnCountKeyOneOn<254) gnCountKeyOneOn++;
            if (gnCountKeyOneOn==gnTimeOnKey) glKeyOneOn=1;
            if (gnCountKeyOneOn==gnTimeLongOnKey) glKeyOneLongOn=1;

            if (gnCountKeyOneOn>gnTimeLongOnKey)
            {
                gnCountKeyOneOn=gnCountKeyOneOn-gnTimeRepeatOnKey;
                glKeyOneOn=1;
            }
        }
        else 
        {
            gnCountKeyOneOn=0;
            glKeyOneOn=0;
            glKeyOneLongOn=0;
}}}
//--------------------------------------------------------------------------------
                                                //    /
                                                //     
                                                //           
void TestKeyStart()
{
    if (++gnCountTimeBetwOnKeyStart>=gnMaxTimeBetwOnKeyStart)   //      
    {
        if (gnNumbOnKeyStart!=0) gnCommand=gnNumbOnKeyStart;    //    <>0 -   
        gnCountTimeBetwOnKeyStart=gnMaxTimeBetwOnKeyStart;      //     
        gnNumbOnKeyStart=0;                                     //   
    }

    if (gnSumKeysOn<=1)
    {
        if ((glCurrKeyStartOn==1) && (glPrevKeyStartOn==1))
        {
            if (gnCountKeyStartOn<254) gnCountKeyStartOn++;
            if (gnCountKeyStartOn==gnTimeOnKey)
            {
                glKeyStartOn=1;
                                                //    <>0     -   
                if ((gnNumbOnKeyStart!=0) && (gnCountTimeBetwOnKeyStart<gnMaxTimeBetwOnKeyStart)) gnNumbOnKeyStart++;
                else gnNumbOnKeyStart=1;        //    
                gnCountTimeBetwOnKeyStart=0;    //     
            }
            if (gnCountKeyStartOn==gnTimeLongOnKey) glKeyStartLongOn=1;
        }
        else if (glPrevKeyStartOn==0)           //         (  1 )
        {
            gnCountKeyStartOn=0;
            glKeyStartOn=0;
            glKeyStartLongOn=0;
}}}
//================================================================================
                                                //     1-Ware Bus ( 
                                                //  "Drivers\TOUCH.C"  CCS PICC   )
//--------------------------------------------------------------------------------
#use standard_io(A)
#use standard_io(B)

BYTE touch_read_byte() {
   BYTE i,data;

   for(i=1;i<=8;++i) {
     output_low(TOUCH_PIN);
     delay_us(14);
     output_float(TOUCH_PIN);
     delay_us(5);
     shift_right(&data,1,input(TOUCH_PIN));
     delay_us(100);
   }
   return(data);
}
//--------------------------------------------------------------------------------
BYTE touch_write_byte(BYTE data) {
   BYTE i;

   for(i=1;i<=8;++i) {
     output_low(TOUCH_PIN);
     delay_us(10);
     if(shift_right(&data,1,0)) {
       output_high(TOUCH_PIN);
       delay_us(10);
         if(!input_state(TOUCH_PIN))
         return(0);
     } else {
       output_low(TOUCH_PIN);
       delay_us(10);
         if(input_state(TOUCH_PIN))
            return(0);
     }
     delay_us(50);
     output_high(TOUCH_PIN);
     delay_us(50);
   }
   return(TRUE);
}
//--------------------------------------------------------------------------------
BYTE touch_present() {
    BOOLEAN present;

    output_low(TOUCH_PIN);
    delay_us(500);
    output_float(TOUCH_PIN);

    delay_us(5);

    if(!input(TOUCH_PIN))
      return(FALSE);
    delay_us(65);
    present=!input(TOUCH_PIN);
    delay_us(240);
    if(present)
      return(TRUE);
    else
      return(FALSE);
}
#use fast_io(A)
#use fast_io(B)
//================================================================================
                                                //    DS18S20,  1
void StartReadTemperature()
{
    bit_Sound_2=1;                              //        (  )

    if (touch_present())
    {
        disable_interrupts(GLOBAL);             //   (      )

        touch_write_byte(0xCC);
        touch_write_byte (0x44);
        output_high(TOUCH_PIN);

        enable_interrupts(GLOBAL);              //  

        gnCountTimeReadTemper=gnTimeReadTemper; //          (     )
    }
    else gnPathReadTemper=4;                    //   ,   ,     "oC"

}
//--------------------------------------------------------------------------------
                                                //    DS18S20,  2
void EndReadTemperature()
{
    bit_Sound_2=1;                              //        (  )

    disable_interrupts(GLOBAL);                 //  

    touch_present();
    touch_write_byte(0xCC);
    touch_write_byte (0xBE);
    gnTemper = touch_read_byte();
    gnZnak = touch_read_byte();

    enable_interrupts(GLOBAL);                  //  

    if (gnZnak==0xFF) gnTemper=0x100-gnTemper;  //   
    gnTemper=gnTemper/2;
}
//================================================================================
                                                //  
void main()
{
init();                                         //  PIC
while(1)                                        //  
{
if (glEnableNextCycle==1)                       // real time,   1000       TMR2
{
//--------------------------------------------------------------------------------
glEnableNextCycle=0;                            //  
EndOutFigure();                                 // "  " (      )
//--------------------------------------------------------------------------------
if (++gnDiv10==10)
{
//--------------------------------------------------------------------------------
                                                //       100 -1
gnDiv10=0;                                      //  

TestKeys();                                     //    

if (gnSTATE==0)                                 //    = " "
{
    TestKeyDec();                               //    
    if (glKeyDecOn==1)
    {
        glKeyDecOn=0;                               //     -    
        if (++gnDec>=10) gnDec=0;                   //   : 0->1->2->...->9->0...
    }
    TestKeyOne();                               //   
    if (glKeyOneOn==1)
    {
        glKeyOneOn=0;                               //    
        if (++gnOne>=10) gnOne=0;
    }
}
//--------------------------------------------------------------------------------
TestKeyStart();                                 //    /
//--------------------------------------------------------------------------------
if (gnSTATE==0)
//--------------------------------------------------------------------------------
{                                               //    0 -  " "
    if ((gnCountWorkingSound==0) && (gnCountWorkingDevice==0))
    {                                           //    =0 -         
        if ((glKeyStartOn==1) && (glCurrKeyStartOn==0))
        {                                       //     (  )
            glKeyStartOn=0;                     //   
            
            gnTimer = gnDec*10 + gnOne;         //   

            gnValConst = gnTimer;               //    (     )
            
            if (gnTimer>0)                      //   <>0
            {
                StartWorkingTimer();            //  
                if (gnTimeWorkingDevice==0)     //       =0
                {                               //    
                    StartWorkingDevice();       // (        )
                }
            }
            else StartDoubleShortWorkingSound();//     
        }
        if (gnCommand==2)                       //      
        {
            gnSTATE=2;                          //    
            gnCommand=0;                        //  
            gnCountTemperOut=gnTimeTemperOut;   //     (      "")
            gnDivToDecSec=0;                    //  
            gnPathReadTemper=0;                 //     5 ,   
        }
    }
    else                                        //       0 -    
    {                                           // ,     :
        glKeyStartOn=0;                         //   ,     
        gnDec=0;
        gnOne=0;
        
        gnCommand=0;                            //   (         )
    }

    if (glKeyStartLongOn==1)                    //      
    {
        if ((gnCountWorkingSound!=0) || (gnCountWorkingDevice!=0))
        {                                       //      
            StopWorkingDevice();                //     (     ),
            StopWorkingSound();                 //       
        }
        else
        {
            TimerOff();                         //    (   )
        }

        glKeyStartOn=0;                         //    
        glKeyStartLongOn=0;
    }

    if (gnNumbOnKeyStart>6) gnSTATE=1;          // (     eeprom  0)
                                                // (,        eeprom -   
                                                //   0x100=256 )

                                                //     100   
}
//--------------------------------------------------------------------------------
else if (gnSTATE==1)                            //    1, " "
{
    if (gnNumbOnKeyStart<4)                     // (     4      )
    {
        if (++gnDiv100==100)                    //  
        {
            gnDiv100=0;                         //  
            if (++gnDiv60==gnNumbSecInMin)      //  
            {                                   //    1 -1
                gnDiv60=0;                      //  
                
                gnTimer--;                      //      1
                gnDec=gnTimer/10;               //       
                gnOne=gnTimer%10;
                
                if (gnTimer==0)                 //     
                {
                    StopWorkingTimer();         //   
                    StartWorkingDevice();       //    
                    StartWorkingSound();        //      
                }
            }
        }
    }
                                                //     100   
    else if (gnNumbOnKeyStart<=8)
    {
        gnDec=207;                              //  ""
        gnOne=208;
        gnTimeOutFigure=100;                    //     ( 100  0     )
        gnTimeNotOutFigure=0;
    }
    else
    {
        gnDec=gnNumbOnKeyStart/10;              //    
        gnOne=gnNumbOnKeyStart%10;
    }

    ClearCountTimeSleep();                      //   
    
    if ((gnCommand>=11) && (gnCommand<=22))     //     11  22
    {
        WriteConstToEEPROM();                   //    eeprom
        ReadConstFromEEPROM();                  //     eeprom
        gnCommand=0;                            //  
        glKeyStartLongOn=1;                     //    
    }

    if (glKeyStartLongOn==1)                    //       
    {
        StopWorkingTimer();                     //  
        glKeyStartOn=0;                         //     
        glKeyStartLongOn=0;
//      TimerOff();                             //       ,
                                                //   ,   ,   
    }
}
//--------------------------------------------------------------------------------
else if (gnSTATE==2)
{                                               //    2, "  "
    ClearCountTimeSleep();
    if ((gnCommand==2) || (gnCountTemperOut==0))//     
    {                                           //           ""
        gnSTATE=0;
        gnCommand=0;
        gnOne=0;
        gnDec=0;
        gnPathReadTemper=0;
        bit_Sound_1=0;                          //      
        bit_Sound_2=0;
        output_bit(pin_Sound_1,0);              //  0   PIC,   
        output_bit(pin_Sound_2,0);
    }
    else
    {
        if (gnCountTimeReadTemper!=0) gnCountTimeReadTemper--;  //      

        if (gnPathReadTemper==0)                //    
        {
            gnPathReadTemper=1;
            gnDec=111;                          //  "oC"
            gnOne=67;
        }
        else if (gnPathReadTemper==1)           // 
        {
            gnPathReadTemper=2;
            StartReadTemperature();             //  
        }
        else if ((gnPathReadTemper==2) && (gnCountTimeReadTemper==0))   //  (    
        {
            gnPathReadTemper=3;
            EndReadTemperature();               //  
        }
        else if (gnPathReadTemper==3)           // 
        {
            gnDec=gnTemper/10;                  //    
            gnOne=gnTemper%10;
            if (gnZnak==0xFF)                   //   
            {
                if (gnDec==0) gnDec=45;         //  0  -10 -  "-"
                else if (gnDec==1) gnDec=124;   //  -10  -20 -  "-|"
                else if (gnDec>1) {gnDec=45; gnOne=45;} //   "--"

            }
            gnPathReadTemper=1;                 //    
        }
        else if (gnPathReadTemper==4)           // ;        (  
        {                                       //    "
            gnPathReadTemper=0;                 //     
        }
    }
}
else gnSTATE=0;
//--------------------------------------------------------------------------------
OutFigure();                                    //     ( 100   ,
                                                // ..       50 Hz
//--------------------------------------------------------------------------------
                                                //   
if (gnCountWorkingSound!=0)                     //        0 ()
{
    if (gnVarSound==1)                          //    : 1  2 -   ,
    {                                           // 3 -   , 4 -    
        //   (   )
        //    100 ,     , , ,   , ...
        //     1   
                                                //    (   TMR1)
        gnTMR1=gnTMR1_01;                       //      TMR1,   1 KHz
        
        if (++gnDivForSound==100) gnDivForSound=0;  //   
        
        if (gnDivForSound<10) glIsSound=0;      // glIsSound=0    
        else if (gnDivForSound<16) glIsSound=1; // glIsSound=1 -  
        else if (gnDivForSound<26) glIsSound=0;
        else if (gnDivForSound<32) glIsSound=1;
        else if (gnDivForSound<42) glIsSound=0;
        else if (gnDivForSound<48) glIsSound=1;
        else glIsSound=0;
    
    }
//--------------------------------------------------------------------------------
    else if (gnVarSound==2)
    {
        //  picmaniac' (   , )
        // 120 ,   33 ,    32-, 1,3,...=1250Hz, 2,4,...=1660Hz
        // ..    (  ),    (3    4  )
        
        if (++gnDivForSound==120) {gnDivForSound=0; gnDivForSound2=7;}
        if (++gnDivForSound2>=7) gnDivForSound2=0;
        
        glIsSound=1;
        if (gnDivForSound<=112)
        {
            if (gnDivForSound2<3) gnTMR1=gnTMR1_02;
            else gnTMR1=gnTMR1_03;
        }
        else glIsSound=0;
    }
//--------------------------------------------------------------------------------
    else if (gnVarSound==3)
    {                                           //      ,   1 KHz
        gnTMR1=gnTMR1_01;                       //  1 KHz
        glIsSound=1;
    }
//--------------------------------------------------------------------------------
    else if (gnVarSound==4)
    {                                           //          ,   1 KHz
        gnTMR1=gnTMR1_01;                       //  1 KHz

        ++gnDivForSound;

        if (gnDivForSound<10) glIsSound=1;
        else if (gnDivForSound<20) glIsSound=0;
        else if (gnDivForSound<30) glIsSound=1;
        else glIsSound=0;
    }
}
//--------------------------------------------------------------------------------
                                                    //    PIC
if (gnCountWorkingSound!=0) ClearCountTimeSleep();  //      -   
if (gnCountWorkingDevice!=0) ClearCountTimeSleep();

if (++gnCountTimeSleep>gnMaxTimeSleep)              //        
{
    gnCountTimeSleep=gnMaxTimeSleep;                //      ,  

    output_bit(pin_Dec,glXorIsCommAnode);           //   (   )      0V
    output_bit(pin_One,glXorIsCommAnode);
    output_b(0);
    
    gnDec=0; gnOne=0;                               //  

    glIsExtInterrutp=0;                             //   (       )
    ext_int_edge(0, H_TO_L);                        //     5V->0V (     )

    clear_interrupt(INT_EXT);                       //    
    enable_interrupts(INT_EXT);                     //   

    while (glIsExtInterrutp==0)                     //     
    {
        sleep();                                    //   
        restart_wdt();                              // ,  WDT
    }

    disable_interrupts(INT_EXT);                    //      ,  
    setup_timer_2(T2_DIV_BY_4,249,1);               //  TMR2
    StartShortWorkingSound();                       //    
}
restart_wdt();                                      //  WDT (   )
//--------------------------------------------------------------------------------
                                                    //     10 -1
if (++gnDivToDecSec==10)
{                                                   
    gnDivToDecSec=0;                                //  

    if (gnSTATE==2)                                 //    " "
    {
        if (gnCountTemperOut!=0) gnCountTemperOut--;    //      
    }
    else                                            // 
    {   
                                                    //  
        if (gnCountWorkingDevice!=0) gnCountWorkingDevice--;    //       0,  ,
        else if (gnSTATE!=1)                        //      " "
            StopWorkingDevice();                    //  

                                                    // ,  
        if (gnCountWorkingSound!=0) gnCountWorkingSound--;
        else StopWorkingSound();
    }
}
//--------------------------------------------------------------------------------
}
}
}
}
                                                    // .